@@ -4,9 +4,16 @@ import os
 
 from dataclasses import dataclass
 
-from dominus.DominusService_pb2 import EntityEntry, Descriptor as PDescriptor, IdentityEntry, TimestampRange, RelationshipEntry
+from dominus.dominus_pb2 import (
+    EntityEntry,
+    Descriptor as PDescriptor,
+    IdentityEntry,
+    TimestampRange,
+    RelationshipEntry,
+)
 
 from bakplane.utils import to_proto_timestamp, to_timestamp, hash_string
+from bakplane.bakplane_pb2 import ErrorEntry
 
 
 @dataclass
@@ -20,13 +27,23 @@ class IngestionSessionContext:
     @staticmethod
     def from_env():
         return IngestionSessionContext(
-            ingestion_session_id=int(os.getenv('INGESTION_SESSION_ID')),
-            entities_output_location=os.getenv('ENTITIES_OUTPUT_LOCATION'),
-            assets_output_location=os.getenv('ASSETS_OUTPUT_LOCATION'),
-            errors_output_location=os.getenv('ERRORS_OUTPUT_LOCATION'),
-            relationships_output_location=os.getenv('RELATIONSHIPS_OUTPUT_LOCATION'),
+            ingestion_session_id=int(os.getenv("INGESTION_SESSION_ID", 0)),
+            entities_output_location=os.getenv(
+                "ENTITIES_OUTPUT_LOCATION", "file://./output/entities/"
+            ),
+            assets_output_location=os.getenv(
+                "ASSETS_OUTPUT_LOCATION", "file://./output/assets/"
+            ),
+            errors_output_location=os.getenv(
+                "ERRORS_OUTPUT_LOCATION", "file://./output/errors/"
+            ),
+            relationships_output_location=os.getenv(
+                "RELATIONSHIPS_OUTPUT_LOCATION",
+                "file://./output/relationships/",
+            ),
         )
 
+
 @dataclass
 class Asset:
     resource_code: str
@@ -49,18 +66,18 @@ class Asset:
                 parts.add(str(self.fields[k]))
             elif k in self.retrieved_by:
                 parts.add(str(self.retrieved_by[k]))
-            elif k == 'effective_start_dt':
+            elif k == "effective_start_dt":
                 parts.add(str(to_timestamp(self.effective_start_dt)))
-            elif k == 'effective_end_dt':
+            elif k == "effective_end_dt":
                 parts.add(str(to_timestamp(self.effective_end_dt)))
-            elif k == 'knowledge_start_dt':
+            elif k == "knowledge_start_dt":
                 if self.knowledge_start_dt:
                     parts.add(str(to_timestamp(self.knowledge_start_dt)))
-            elif k == 'knowledge_end_dt':
+            elif k == "knowledge_end_dt":
                 if self.knowledge_end_dt:
                     parts.add(str(to_timestamp(self.knowledge_end_dt)))
 
-        return hash_string(''.join(sorted(list(parts))))
+        return hash_string("".join(sorted(list(parts))))
 
     @property
     def key_hash(self):
@@ -82,17 +99,27 @@ class Asset:
         for k in sorted_fields:
             parts.append(str(self.fields[k]))
 
-        parts.append(self.effective_start_dt.strftime('%Y-%m-%d') + ' 00:00:00.00000')
-        parts.append(self.effective_end_dt.strftime('%Y-%m-%d') + ' 00:00:00.00000')
+        parts.append(
+            self.effective_start_dt.strftime("%Y-%m-%d") + " 00:00:00.00000"
+        )
+        parts.append(
+            self.effective_end_dt.strftime("%Y-%m-%d") + " 00:00:00.00000"
+        )
 
         if self.knowledge_start_dt is None:
             self.knowledge_start_dt = datetime.datetime.utcnow()
 
         if self.knowledge_end_dt is None:
-            self.knowledge_end_dt = datetime.datetime(year=3000, month=12, day=31)
+            self.knowledge_end_dt = datetime.datetime(
+                year=3000, month=12, day=31
+            )
 
-        parts.append(self.knowledge_start_dt.strftime('%Y-%m-%d') + ' 00:00:00.00000')
-        parts.append(self.knowledge_end_dt.strftime('%Y-%m-%d') + ' 00:00:00.00000')
+        parts.append(
+            self.knowledge_start_dt.strftime("%Y-%m-%d") + " 00:00:00.00000"
+        )
+        parts.append(
+            self.knowledge_end_dt.strftime("%Y-%m-%d") + " 00:00:00.00000"
+        )
 
         parts.append(self.resource_code)
         parts.append(str(ctx.ingestion_session_id))
@@ -100,7 +127,7 @@ class Asset:
         parts.append(str(self.key_hash))
         parts.append(str(self.payload_hash))
 
-        return '|'.join(parts)
+        return "|".join(parts)
 
 
 @dataclass
@@ -115,11 +142,9 @@ class Identity:
         for i in sorted(self.attributes.keys()):
             c.append(i + self.attributes[i])
 
-        return hash_string(''.join([
-            self.perspective,
-            self.entity_type,
-            ''.join(c)
-        ]))
+        return hash_string(
+            "".join([self.perspective, self.entity_type, "".join(c)])
+        )
 
     def to_proto(self) -> IdentityEntry:
         return IdentityEntry(
@@ -142,8 +167,12 @@ class Descriptor:
         return PDescriptor(
             attribute_code=self.attribute,
             value=self.value,
-            effective_dating=[TimestampRange(start_dt=to_proto_timestamp(self.effective_start_dt),
-                                             end_dt=to_proto_timestamp(self.effective_end_dt))]
+            effective_dating=[
+                TimestampRange(
+                    start_dt=to_proto_timestamp(self.effective_start_dt),
+                    end_dt=to_proto_timestamp(self.effective_end_dt),
+                )
+            ],
         )
 
 
@@ -172,12 +201,24 @@ class Relationship:
             source_identity_uid=self.source_identity_uid,
             target_identity_uid=self.target_identity_uid,
             relationship_type_code=self.relationship_type,
-            effective_dating=[TimestampRange(start_dt=to_proto_timestamp(self.effective_start_dt),
-                                             end_dt=to_proto_timestamp(self.effective_end_dt))],
+            effective_dating=[
+                TimestampRange(
+                    start_dt=to_proto_timestamp(self.effective_start_dt),
+                    end_dt=to_proto_timestamp(self.effective_end_dt),
+                )
+            ],
         )
 
 
 @dataclass
 class Error:
-    pass
+    message: str
+    stacktrace: str = None
+    context: typing.Dict[str, str] = None
 
+    def to_proto(self) -> ErrorEntry:
+        return ErrorEntry(
+            message=self.message,
+            stacktrace=self.stacktrace,
+            context=self.context,
+        )
